from django.contrib import admin

from .models import Rate


class RateAdmin(admin.ModelAdmin):
    list_display = ('currency', 'value', 'last_update', 'backend')
    list_filter = ('currency', )
    ordering = ('currency', )
    readonly_fields = ('backend', )

    def last_update(self, instance):
        return instance.backend.last_update


admin.site.register(Rate, RateAdmin)



# -*- coding: utf-8 -*-
from rest_framework.serializers import DecimalField, ModelSerializer

from djmoney.models.fields import MoneyField as ModelField
from djmoney.money import Money
from djmoney.utils import MONEY_CLASSES, get_currency_field_name


class MoneyField(DecimalField):
    """
    Treats ``Money`` objects as decimal values in representation and
    does decimal's validation during transformation to native value.
    """

    def to_representation(self, obj):
        """
        When ``field_currency`` is not in ``self.validated_data`` then ``obj`` is an instance of ``Decimal``, otherwise
        it is ``Money``.
        """
        if isinstance(obj, MONEY_CLASSES):
            obj = obj.amount
        return super(MoneyField, self).to_representation(obj)

    def to_internal_value(self, data):
        if isinstance(data, MONEY_CLASSES):
            amount = super(MoneyField, self).to_internal_value(data.amount)
            return Money(amount, data.currency)
        return super(MoneyField, self).to_internal_value(data)

    def get_value(self, data):
        amount = super(MoneyField, self).get_value(data)
        currency = data.get(get_currency_field_name(self.field_name), None)
        if currency:
            return Money(amount, currency)
        return amount


def register_money_field():
    ModelSerializer.serializer_field_mapping[ModelField] = MoneyField



import json
from decimal import Decimal

from django.db.transaction import atomic
from django.utils.http import urlencode

from djmoney import settings
from djmoney._compat import parse_qsl, urlopen, urlparse, urlunparse

from ..models import ExchangeBackend, Rate


class BaseExchangeBackend(object):
    name = None
    url = None

    def get_rates(self, **kwargs):
        """
        Returns a mapping <currency>: <rate>.
        """
        raise NotImplementedError

    def get_url(self, **params):
        """
        Updates base url with provided GET parameters.
        """
        parts = list(urlparse(self.url))
        query = dict(parse_qsl(parts[4]))
        query.update(params)
        parts[4] = urlencode(query)
        return urlunparse(parts)

    def get_params(self):
        """
        Default GET parameters for the request.
        """
        return {}

    def get_response(self, **params):
        url = self.get_url(**params)
        response = urlopen(url)
        return response.read()

    def parse_json(self, response):
        if isinstance(response, bytes):
            response = response.decode('utf-8')
        return json.loads(response, parse_float=Decimal)

    @atomic
    def update_rates(self, base_currency=settings.BASE_CURRENCY, **kwargs):
        """
        Updates rates for the given backend.
        """
        backend, _ = ExchangeBackend.objects.update_or_create(name=self.name, defaults={'base_currency': base_currency})
        backend.clear_rates()
        params = self.get_params()
        params.update(base_currency=base_currency, **kwargs)
        Rate.objects.bulk_create([
            Rate(currency=currency, value=value, backend=backend)
            for currency, value in self.get_rates(**params).items()
        ])


class SimpleExchangeBackend(BaseExchangeBackend):
    """
    Simple backend implementation.
    Assumes JSON response with `rates` key inside.
    """

    def get_rates(self, **params):
        response = self.get_response(**params)
        return self.parse_json(response)['rates']



from django.core.exceptions import ImproperlyConfigured

from djmoney import settings

from .base import SimpleExchangeBackend


class FixerBackend(SimpleExchangeBackend):
    name = 'fixer.io'

    def __init__(self, url=settings.FIXER_URL, access_key=settings.FIXER_ACCESS_KEY):
        if access_key is None:
            raise ImproperlyConfigured('settings.FIXER_ACCESS_KEY should be set to use FixerBackend')
        self.url = url
        self.access_key = access_key

    def get_params(self):
        return {'access_key': self.access_key}



# -*- coding: utf-8 -*-
from django.forms import MultiWidget, Select, TextInput

from ..settings import CURRENCY_CHOICES


__all__ = ('MoneyWidget',)


class MoneyWidget(MultiWidget):

    def __init__(self, choices=CURRENCY_CHOICES, amount_widget=TextInput, currency_widget=None, default_currency=None,
                 *args, **kwargs):
        self.default_currency = default_currency
        if not currency_widget:
            currency_widget = Select(choices=choices)
        widgets = (amount_widget, currency_widget)
        super(MoneyWidget, self).__init__(widgets, *args, **kwargs)

    def decompress(self, value):
        if value is not None:
            return [value.amount, value.currency]
        return [None, self.default_currency]



from django.utils.module_loading import import_string

from ..base import BaseExchangeCommand


class Command(BaseExchangeCommand):
    help = 'Updates exchange rates.'

    def handle(self, *args, **options):
        backend = import_string(options['backend'])()
        backend.update_rates()
        self.success('Successfully updated rates from %s' % backend.name)



# Generated by Django 2.0.4 on 2018-04-03 08:42

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ExchangeBackend',
            fields=[
                ('name', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('base_currency', models.CharField(max_length=3)),
            ],
        ),
        migrations.CreateModel(
            name='Rate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('currency', models.CharField(max_length=3)),
                ('value', models.DecimalField(decimal_places=6, max_digits=20)),
                ('backend', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='exchange.ExchangeBackend', related_name='rates')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='rate',
            unique_together={('currency', 'backend')},
        ),
    ]



# -*- coding: utf-8 -*-
from django.apps import AppConfig


class MoneyConfig(AppConfig):
    name = 'djmoney'

    def ready(self):
        try:
            from .contrib.django_rest_framework import register_money_field

            register_money_field()
        except ImportError:
            pass
        from djmoney.admin import setup_admin_integration

        setup_admin_integration()



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.module_loading import import_string

from djmoney.settings import EXCHANGE_BACKEND

from .exceptions import MissingRate


class ExchangeBackend(models.Model):
    name = models.CharField(max_length=255, primary_key=True)
    last_update = models.DateTimeField(auto_now=True)
    base_currency = models.CharField(max_length=3)

    def __str__(self):
        return self.name

    def clear_rates(self):
        self.rates.all().delete()


class Rate(models.Model):
    currency = models.CharField(max_length=3)
    value = models.DecimalField(max_digits=20, decimal_places=6)
    backend = models.ForeignKey(ExchangeBackend, on_delete=models.CASCADE, related_name='rates')

    class Meta:
        unique_together = (('currency', 'backend'), )


def get_default_backend_name():
    return import_string(EXCHANGE_BACKEND).name


def get_one():
    """
    For SQLite it is required to cast value to NUMERIC type, otherwise integer division will be used.
    """
    if settings.DATABASES['default']['ENGINE'] == 'django.db.backends.sqlite3':
        return '1::NUMERIC'
    return 1


def get_rate(source, target, backend=None):
    """
    Returns an exchange rate between source and target currencies.
    Converts exchange rate on the DB side if there is no backends with given base currency.
    Uses data from the default backend if the backend is not specified.
    """
    if backend is None:
        backend = get_default_backend_name()
    if source == target:
        return 1
    try:
        forward = models.Q(currency=target, backend__base_currency=source)
        reverse = models.Q(currency=source, backend__base_currency=target)
        return Rate.objects.annotate(
            rate=models.Case(
                models.When(forward, then=models.F('value')),
                models.When(reverse, then=models.Value(get_one()) / models.F('value')),
            )
        ).get(forward | reverse, backend=backend).rate
    except Rate.DoesNotExist:
        raise MissingRate('Rate %s -> %s does not exist' % (source, target))


def convert_money(value, currency):
    if 'djmoney.contrib.exchange' not in settings.INSTALLED_APPS:
        raise ImproperlyConfigured(
            "You have to add 'djmoney.contrib.exchange' to INSTALLED_APPS in order to use currency exchange"
        )
    amount = value.amount * get_rate(value.currency, currency)
    return value.__class__(amount, currency)



from django.utils.module_loading import import_string

from djmoney.contrib.exchange.models import Rate

from ..base import BaseExchangeCommand


class Command(BaseExchangeCommand):
    help = 'Clears exchange rates.'

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            '--all', action='store_true', dest='all',
            help='Clear rates for all backends.',
            required=False,
            default=False,
        )

    def handle(self, *args, **options):
        if options['all']:
            Rate.objects.all().delete()
            message = 'Successfully cleared all rates'
        else:
            backend = import_string(options['backend'])
            Rate.objects.filter(backend=backend.name).delete()
            message = 'Successfully cleared rates for %s' % backend.name
        self.success(message)



from django.core.management.base import BaseCommand

from djmoney import settings
from djmoney._compat import get_success_style


class BaseExchangeCommand(BaseCommand):
    """
    Basic command for exchange rates manipulation.
    Provides ``backend`` argument.
    """

    def add_arguments(self, parser):
        parser.add_argument(
            '-b', '--backend',
            action='store',
            dest='backend',
            help='Importable string for custom exchange rates backend.',
            required=False,
            default=settings.EXCHANGE_BACKEND,
        )

    def success(self, message):
        style = get_success_style(self.style)
        self.stdout.write(style(message))



from django.core.exceptions import ImproperlyConfigured

from djmoney import settings

from .base import SimpleExchangeBackend


class OpenExchangeRatesBackend(SimpleExchangeBackend):
    name = 'openexchangerates.org'
    url = settings.OPEN_EXCHANGE_RATES_URL

    def __init__(self, url=settings.OPEN_EXCHANGE_RATES_URL, access_key=settings.OPEN_EXCHANGE_RATES_APP_ID):
        if access_key is None:
            raise ImproperlyConfigured(
                'settings.OPEN_EXCHANGE_RATES_APP_ID should be set to use OpenExchangeRatesBackend'
            )
        self.url = url
        self.access_key = access_key

    def get_params(self):
        return {'app_id': self.access_key}
